/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit :  NetOptim                                                   */
/*    Fichier :  neto.h                                                     */
/*                                                                          */
/*    (c) copyright 1991 Laboratoire MASI equipe CAO & VLSI                 */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : N. Dictus                             le : 10/04/1992     */
/*                                                                          */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*    Modifie par :                                     le : ../../....     */
/*                                                                          */
/****************************************************************************/

long nbChgt;

befig_list *VHB_HEDFIG;

Alist *memberAlist();
chain_list *cellLoadEnvel();
Alist *cellLoadIn();
chain_list *cellLoadOut();
int cellLoadPolarity();
int cellLoadType();
long cellLoadSurface();
long cellLoadFanout();
long cellLoadCin();
long cellLoadR();
long cellLoadTp();
chain_list *appendChain();
void createCell();
short cellLoad();
void cellInit();
